/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.util.monolog.wrapper.remote.lib;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.Handler;
import org.objectweb.util.monolog.api.Level;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.MonologFactory;
import org.objectweb.util.monolog.api.TopicalLogger;
import org.objectweb.util.monolog.file.monolog.PropertiesConfAccess;
import org.objectweb.util.monolog.wrapper.remote.api.LoggerInfo;
import org.objectweb.util.monolog.wrapper.remote.api.MonologFactoryMBean;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * @author S.Chassande-Barrioz
 */
public class MonologFactoryMBeanImpl implements MonologFactoryMBean {

    private MonologFactory mf;
    
    public MonologFactoryMBeanImpl()  {
        mf = Monolog.getMonologFactory();
    }    
    
    public MonologFactoryMBeanImpl(MonologFactory mf)  {
        this.mf = mf;
    }
    
    public boolean defineLevel(String name, int value)  {
        return mf.defineLevel(name, value) != null;
    }

    public boolean defineLevel(String name, String value)
             {
        return mf.defineLevel(name, value) != null;
    }

    public void removeLevel(String name)  {
        mf.removeLevel(name);
    }

    public Level getLevel(String name)  {
        return mf.getLevel(name);
    }

    public Level getLevel(int value)  {
        return mf.getLevel(value);
    }

    public Level[] getLevels()  {
        return mf.getLevels();
    }

    public int compareTo(String levelname1, String levelname2)
             {
        Level l1 = mf.getLevel(levelname1);
        if (l1 == null) {
            return Integer.MAX_VALUE;
        }
        Level l2 = mf.getLevel(levelname2);
        if (l1 == null) {
            return Integer.MIN_VALUE;
        }
        return l1.compareTo(l2);
    }

    public boolean createHandler(String hn, String handlertype)
             {
        return mf.createHandler(hn, handlertype) != null;
    }

    public boolean removeHandler(String handlername)  {
        return mf.removeHandler(handlername) == null;
    }

    public String[] getHandlerNames()  {
        Handler[] hs = mf.getHandlers();
        String[] hns = new String[hs.length];
        for(int i=0; i<hs.length; i++) {
            hns[i] = hs[i].getName();
        }
        return hns;
    }

    public Map getHandlerAttributes(String handlername)  {
        Handler h = mf.getHandler(handlername);
        String[] ans = h.getAttributeNames();
        Map m = new HashMap(ans.length);
        for(int i=0; i<ans.length; i++) {
            m.put(ans[i], h.getAttribute(ans[i]));
        }
        return m;
    }

    public Map getAllHandlerAttributes()
             {
        Handler[] hs = mf.getHandlers();
        Map m = new HashMap(hs.length);
        for(int i=0; i<hs.length; i++) {
            m.put(hs[i], getHandlerAttributes(hs[i].getName()));
        }
        return m;
    }

    public void setHandlerAttribute(String handlername, String attributeName,
            String value)  {
        Handler h = mf.getHandler(handlername);
        if (h == null) {
            return;
        }
        h.setAttribute(attributeName, value);
    }

    public LoggerInfo getLogger(String loggername)  {
        return new LoggerInfo((TopicalLogger) mf.getLogger(loggername));
    }

    public LoggerInfo getLogger(String loggername, String resourceBundleName)
             {
        return new LoggerInfo((TopicalLogger) 
                mf.getLogger(loggername, resourceBundleName));
    }

    public String getResourceBundleName()  {
        return mf.getResourceBundleName();
    }

    public void setResourceBundleName(String resourceBundleName)
             {
        mf.setResourceBundleName(resourceBundleName);
    }

    public LoggerInfo[] getLoggers()  {
        Logger[] ls = mf.getLoggers();
        LoggerInfo[] lis = new LoggerInfo[ls.length];
        for(int i=0; i<ls.length; i++) {
            lis[i] = new LoggerInfo((TopicalLogger) ls[i]);
        }
        return lis;
    }

    public void addHandlerToLogger(String handlername, String loggerName)
             {
        Handler h = mf.getHandler(handlername);
        if (h == null) {
            return;
        }
        TopicalLogger l = (TopicalLogger) mf.getLogger(loggerName);
        try {
            l.addHandler(h);
        } catch (Exception e) {
            return;
        }
    }

    public void removeHandlerFromLogger(String handlerName, String loggerName)
             {
        Handler h = mf.getHandler(handlerName);
        if (h == null) {
            return;
        }
        TopicalLogger l = (TopicalLogger) mf.getLogger(loggerName);
        try {
            l.removeHandler(h);
        } catch (Exception e) {
            return;
        }
    }

     public void removeAllHandlersFromLogger(String loggerName)  {
        TopicalLogger l = (TopicalLogger) mf.getLogger(loggerName);
        try {
            l.removeAllHandlers();
        } catch (Exception e) {
            return;
        }
    }

    public void setAdditivity(boolean a, String loggerName)  {
        TopicalLogger l = (TopicalLogger) mf.getLogger(loggerName);
        l.setAdditivity(a);
    }

    public void addTopicToLogger(String topic, String loggerName) {
        TopicalLogger l = (TopicalLogger) mf.getLogger(loggerName);
        try {
            l.addTopic(topic);
        } catch (Exception e) {
            return;
        }
    }

    public void removeTopicFromLogger(String topic, String loggerName) {
        TopicalLogger l = (TopicalLogger) mf.getLogger(loggerName);
        try {
            l.removeTopic(topic);
        } catch (Exception e) {
            return;
        }
    }

    public Properties getMonologProperties()  {
        Properties p = new Properties();
        try {
            PropertiesConfAccess.store(p, mf);
        } catch (Exception e) {
            return null;
        }
        return p;
    }
    
}
